#include <GL/glut.h>
#include <time.h>  

/* Some <math.h> files do not define M_PI... */
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include "GameObj.h"
#include "Bullet.h"
#include "Ship.h"
#include "GameSession.h"
#include "utility.h"

#ifdef _WIN32
#define drand48() (((float) rand())/((float) RAND_MAX))
#define srand48(x) (srand((x)))
#endif

/** Global Variable storing the Game Session */
GameSession Game;

/** @name GLUT callbacks
@memo These functions are called back by GLUT main loop
	*/
//@{

/** Glut Callback on standard key input.
Mainly to get the space key shoot and esc quitting. */
void Key(unsigned char key, int x, int y)
{
  switch (key) {
  case 27:
    exit(0);
  case ' ':
	 Game.Shoot();	 
  break;
  }
}

/** Glut Callback on pression of arrow keys.
It should start the thrust (up arrow), 
the rotation (left/right arrow) and side thrust 
(alt+left/right) of  the ship.
Version History {\small 
\begin{itemize}
\item 0.1 First Release.
\item 0.2s
Added side movemente management.
\end{itemize}
}
*/
void  SpecialKey(int key, int x, int y)
{

  int km = glutGetModifiers();
	switch (key) {
  case GLUT_KEY_LEFT: 
	  if(km & GLUT_ACTIVE_ALT) Game.S->StartStrafeLeft();
			else Game.S->StartLeft();
	  break;
	case GLUT_KEY_RIGHT: 
		if(km & GLUT_ACTIVE_ALT) Game.S->StartStrafeRight();
			else  Game.S->StartRight();
	  break;
  case GLUT_KEY_UP: 
	  Game.S->ThrustOn();
	  break;
	}
}
/** Glut Callback on release of arrow keys.
It should stop the (side and back) thrust and the rotation of the ship.

Version History {\small 
\begin{itemize}
\item 0.1 First Release.
\item 0.2s
Added side movemente management
\end{itemize}
}
*/
void SpecialKeyUp(int key, int x, int y)
{
  switch (key) {
  case GLUT_KEY_LEFT: 
	  Game.S->StopStrafeLeft();
		Game.S->StopLeft();
	  break;
  case GLUT_KEY_RIGHT: 
	  Game.S->StopStrafeRight();
		Game.S->StopRight();
	  break;
  case GLUT_KEY_UP: 
	  Game.S->ThrustOff();
	  break;
	}
}

/** Opengl Initialization.
Initialization of standard things for Shaded display:
\begin{itemize}
\item Light Setup (position, enabling and light model)
\item Enable Culling and and disable double face materials. 
Note that you should pay attention to always draw faces 
with correct vertex ordering.
\item Enable Depth test and color_material (to change material with a simple glColor)
\end{itemize}
Version History {\small 
\begin{itemize}
\item 0.1 First Release (almost nothing here.
\item 0.2 Added light setup, depth test, culling disable and double sided lighting
\item 0.3 Enabled Culling and Disabled double sided lighting (Better performance!)	
\end{itemize}
}

*/
void myGLInit(void)
{
	float lp[4]={100,100,200,0};
	glLightfv(GL_LIGHT0,GL_POSITION,lp);
  glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	//glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,1);
	//glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_CULL_FACE);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);
} 

/** Glut Callback on Window Reshape.
It updates the Projection Matrix and the viewport and stores the
new window size into memebers of GameSession object. Updated in V2
to correctly handle non square windows and always show the whole 
(square) playfield.
*/
void Reshape(GLsizei w, GLsizei h)
{ 
  Game.ScreenW=w;
  Game.ScreenH=h;
	float wr=float(w)/h;
  float hr=float(h)/w;

	if(w>h) hr=1;
	else wr=1;
  int VL = -Game.SpaceSize*wr;
  int VR =  Game.SpaceSize*wr;
  int VB = -Game.SpaceSize*hr;
  int VT =  Game.SpaceSize*hr;
  //int VB = -Game.SpaceSize;
  //int VT =  Game.SpaceSize;
  glViewport(0,0,w,h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(VL,VR,VB,VT,-100,100);
  glMatrixMode(GL_MODELVIEW);
}		

/** Glut Idle Callback .
It is continuously called when events are not
being received. It post a refresh request in order to make the animation
as smoother as possible. It is enabled/disable by the Visible Callback
@see Visible
*/

void Idle(void)
{
	glutPostRedisplay();
}
/** Glut Callback on Window visibility.
It is called when the glut window change its visiblitiy status.
When the window is hidden we should stop the idle callback (and viceversa).
*/
void Visible(int vis)
{
 if (vis == GLUT_VISIBLE)
      glutIdleFunc(Idle);
 else
      glutIdleFunc(NULL);
}


/** Glut Callback to Draw everything.
It clear the screen, call the GameSession 
Update() and Draw() and swap the buffers.
Keep also track of the FPS rate.
*/
void Display(void)
{ 
  static int lasttime;
  int dt;
  dt=glutGet(GLUT_ELAPSED_TIME)-lasttime;
  lasttime=glutGet(GLUT_ELAPSED_TIME);

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glPrintf(5,Game.ScreenH-20,20,"fps %3.1f\n",FPS(dt));
  glutGet(GLUT_ELAPSED_TIME);
  Game.Update(dt);
  Game.Draw();
  glutSwapBuffers();
}
//@}

/** The Classical Main Function
It has to setup all the glut callbacks, initialize Openglstatus
and start the glutMainLoop.
*/
int main(int argc, char** argv)
{ 
  glutInit(&argc, argv);
  glutInitWindowSize(512, 512);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );
  glutCreateWindow("Asteroids");

  glutReshapeFunc(Reshape);
  glutDisplayFunc(Display);
  glutIdleFunc(Idle);
	glutVisibilityFunc(Visible);
  glutKeyboardFunc(Key);	
  glutSpecialFunc(SpecialKey);	
  glutSpecialUpFunc(SpecialKeyUp);	
    
  myGLInit(); 
  srand48(time(NULL));	
	Game.Start();
  glutMainLoop();
  return 0;             
}
